@@ -25,6 +25,7 @@
 #ifndef EVENTSTR_H
 #define EVENTSTR_H
 
+#include "inputstr.h"
 #include <events.h>
 /**
  * @file events.h
@@ -77,6 +78,14 @@ enum EventType {
     ET_Internal = 0xFF          /* First byte */
 };
 
+/**
+ * How a DeviceEvent was provoked
+ */
+enum DeviceEventSource {
+  EVENT_SOURCE_NORMAL = 0, /**< Default: from a user action (e.g. key press) */
+  EVENT_SOURCE_FOCUS, /**< Keys or buttons previously down on focus-in */
+};
+
 /**
  * Used for ALL input device events internal in the server until
  * copied into the matching protocol event.
@@ -123,6 +132,8 @@ struct _DeviceEvent {
     int corestate;    /**< Core key/button state BEFORE the event */
     int key_repeat;   /**< Internally-generated key repeat event */
     uint32_t flags;   /**< Flags to be copied into the generated event */
+    uint32_t resource; /**< Touch event resource, only for TOUCH_REPLAYING */
+    enum DeviceEventSource source_type; /**< How this event was provoked */
 };
 
 /**
@@ -175,7 +186,7 @@ struct _DeviceChangedEvent {
     struct {
         uint32_t min;           /**< Minimum value */
         uint32_t max;           /**< Maximum value */
-        double value;           /**< Current value */;
+        double value;           /**< Current value */
         /* FIXME: frac parts of min/max */
         uint32_t resolution;    /**< Resolution counts/m */
         uint8_t mode;           /**< Relative or Absolute */
@@ -189,7 +200,7 @@ struct _DeviceChangedEvent {
     } keys;
 };
 
-#if XFreeXDGA
+#ifdef XFreeXDGA
 /**
  * DGAEvent, used by DGA to intercept and emulate input events.
  */
@@ -276,7 +287,7 @@ union _InternalEvent {
     DeviceChangedEvent changed_event;
     TouchOwnershipEvent touch_ownership_event;
     BarrierEvent barrier_event;
-#if XFreeXDGA
+#ifdef XFreeXDGA
     DGAEvent dga_event;
 #endif
     RawDeviceEvent raw_event;
